/*
 * http_server_ssi.h
 *
 *  Created on: 27.8.2012
 *      Author: velek
 */

#ifndef HTTP_SERVER_SSI_H_
#define HTTP_SERVER_SSI_H_

#include <stdint.h>
#include <stddef.h>

/**
 * Signature of the callback used by SSI processor to receive
 * data for tags.
 *
 * @param tag A SSI tag to be replaced
 * @param buffer	A pointer to dynamically(malloc) created buffer with
 * 				 	data replacing the tag
 * @return Size of created buffer, zero on error
 */
typedef
size_t (*SSI_TAG_HANDLER)(const char * tag, uint8_t ** buffer);

/**
 * Process a SSI file.
 *
 * @param ssi_file_content	Pointer to data with SSI content
 * @param ssi_file_size		Size of data
 * @param ssi_tag_handler	A callback function to handle tags in SSI file
 *
 * @return NULL on error, MHD_Response on success
 */
struct MHD_Response * http_server_ssi(const uint8_t * ssi_file_content,
		size_t ssi_file_size, SSI_TAG_HANDLER ssi_tag_handler);

#endif /* HTTP_SERVER_SSI_H_ */
