#if 1
#include "http_server.h"
//------------------------------------------------------------------------------------------------
const char *ssi_file =
		"<html><body>This is a SSI page. "
				"On the server, the page contains &lt;!--#echo--&gt; tag."
				"It is replaced by the echo handler with result: <!--#echo--></body></html>";

size_t ssi_tag_handler(const char * tag, uint8_t ** buffer)
{
	(void) tag;
	const char * hello_wrld = "Hello world!";
	*buffer = malloc(strlen(hello_wrld) + 1);
	strcpy(*buffer, hello_wrld);
	return strlen(hello_wrld);
}

int ssi_page(struct MHD_Connection *connection, const char * method,
		const char *upload_data, size_t *upload_data_size, void **con_cls)
{
	int ret;
	struct MHD_Response *response;

	response = http_server_ssi(ssi_file, strlen(ssi_file), ssi_tag_handler);
	ret = MHD_queue_response(connection, MHD_HTTP_OK, response);
	MHD_destroy_response(response);

	return ret;
}



const char *cgi_file =
		"<html><body>This page was completely generated from a function.</body></html>";

size_t cgi_handler(uint8_t ** buffer)
{
	*buffer = malloc(strlen(cgi_file) + 1);
	strcpy(*buffer, cgi_file);
	return strlen(cgi_file);
}

int cgi_page(struct MHD_Connection *connection, const char * method,
		const char *upload_data, size_t *upload_data_size, void **con_cls)
{
	int ret;
	struct MHD_Response *response;

	response = http_server_cgi(cgi_handler);
	ret = MHD_queue_response(connection, MHD_HTTP_OK, response);
	MHD_destroy_response(response);

	return ret;
}

const char *html_file =
		"<html><body>This page is a standard html file.</body></html>";

int html_page(struct MHD_Connection *connection, const char * method,
		const char *upload_data, size_t *upload_data_size, void **con_cls)
{
	int ret;
	struct MHD_Response *response;

	response = http_server_send_data(html_file, strlen(html_file), true);
	ret = MHD_queue_response(connection, MHD_HTTP_OK, response);
	MHD_destroy_response(response);

	return ret;
}

const char *upload_page =
		"<html><body>\n\
		                       Upload firmware file, please!<br>\n\
		                       <form action=\"/fwupgrade\" method=\"post\" enctype=\"multipart/form-data\">\n\
		                       <input name=\"file\" type=\"file\">\n\
		                       <input type=\"submit\" value=\" Send \"></form>\n\
		Or try <a href = \"ssi\"> SSI page </a><br><a href = \"cgi\"> CGI page </a> <br><a href = \"html\"> HTML page </a>\
		                       </body></html>";

int root_page(struct MHD_Connection *connection, const char * method,
		const char *upload_data, size_t *upload_data_size, void **con_cls)
{
	int ret;
	struct MHD_Response *response;

	response = http_server_send_data(upload_page, strlen(upload_page), true);
	ret = MHD_queue_response(connection, MHD_HTTP_OK, response);
	MHD_destroy_response(response);

	return ret;
}

static int upgrade_post(void *coninfo_cls, enum MHD_ValueKind kind,
		const char *key, const char *filename, const char *content_type,
		const char *transfer_encoding, const char *data, uint64_t off,
		size_t size)
{

	return MHD_YES;
}

void fwupgrade_completed_callback(struct MHD_Connection *connection,
		void **con_cls, enum MHD_RequestTerminationCode toe)
{
	if ((NULL != *con_cls) && !HTTP_IS_NEW_CONNECTION(*con_cls))
	{
		struct MHD_PostProcessor *con_info = *con_cls;
        MHD_destroy_post_processor (con_info);
		*con_cls = NULL;
	}
}

const char *completepage =
		"<html><body>The upload has been completed.</body></html>";

int fwupgrade(struct MHD_Connection *connection, const char * method,
		const char *upload_data, size_t *upload_data_size, void **con_cls)
{
	int ret;
	struct MHD_Response *response;

	if (HTTP_IS_NEW_CONNECTION(*con_cls))
	{
		struct MHD_PostProcessor *postprocessor;
		postprocessor = MHD_create_post_processor(connection, 512, upgrade_post,
				(void *) NULL );
		if (NULL == postprocessor)
		{
			return MHD_NO;
		}
		*con_cls = (void *) postprocessor;
	}


	if (0 != *upload_data_size)
	{
		struct MHD_PostProcessor *postprocessor = *con_cls;
		MHD_post_process(postprocessor, upload_data, *upload_data_size);
		*upload_data_size = 0;

		return MHD_YES;
	}
	else
	{

		response = http_server_send_data(completepage, strlen(completepage),
				true);
		ret = MHD_queue_response(connection, MHD_HTTP_OK, response);
		MHD_destroy_response(response);
		return ret;
	}
}
//------------------------------------------------------------------------------------------------


static void www_server_init(void)
{
	http_server_set_credentials("admin", "admin");

	http_server_add_url_handler(NULL , "/ssi", ssi_page, NULL);
	http_server_add_url_handler(NULL , "/cgi", cgi_page, NULL);
	http_server_add_url_handler(NULL , "/html", html_page, NULL);
	http_server_add_url_handler(NULL , "/", root_page, NULL);
	http_server_add_url_handler(NULL ,"/fwupgrade", fwupgrade, fwupgrade_completed_callback);

}
#endif
